/***************************************************************************
                          rtp.c - construction/destruction of rtp sessions
                             -------------------
    begin                : ven mar  30
    copyright            : (C) 2001 by Simon MORLAT
    email                : simon.morlat@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "rtp.h"
#include "rtp_private.h"
#include "rtp_ptypes.h"
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <sys/errno.h>

#define MAX(a,b) ((a < b)? b:a)

int rtp_length[MAX_PROFILE+1]={
                            172,/*PCMU*/
                            0,
                            0,
                            45, /*GSM*/
                            19, /*LPC10*/
                            0,
                            0,
                            0,
                            172 /*PCMA*/
                            };


char *errnum[RTP_MAX_ERRORS]={"-22","Invalid argument.",
			      "-11","Cannot create more sockets.",
			      "-301","Rtp session not initialised.",
			      "-223","Functionnality not yet implemented.",
			      "-300","Invalid internet address.",
			      "-226","Internet address already in use.",
			      "-302","The buffer is too small.",
                  "-303","The session was closed by remote host.",
			      "-95","Option or format not supported.",
				  "-304","Time out.",
				  "-305","lprtplib major faillure."
};


RtpSession * rtp_create_session(int direction)
{
	RtpSession *obj;

  if (!((direction==RTP_RECV) || (direction==RTP_SEND) || (direction==RTP_DUPLEX)))
    {
      return(NULL);
    }
  obj=(RtpSession*)calloc(1,sizeof(RtpSession));
  memset(obj,0,sizeof(RtpSession));
  obj->direction=direction;
  obj->timeout=NULL;
  return(obj);
}

char* rtp_perror(int error)
{
  int i;
  static char *errstr="Unknown error      ";

  for(i=0;i<RTP_MAX_ERRORS;i+=2)
    {
      if (atoi(errnum[i])==error) return(errnum[i+1]);
    }
	//sprintf(errstr+14,"%i",error);
  return(errstr);
}

int rtp_set_parameters(RtpSession *handle,rtp_params_t *params)
{
  int err,j=0;
  int option=1;
  int addrlen;

  memcpy(&handle->params,params,sizeof(rtp_params_t));
  if (handle->params.stacksize==0) handle->params.stacksize=RTP_STACKSIZE;
  if ((handle->params.jitt_comp>handle->params.stacksize)
      || (handle->params.jitt_comp==0)) return(-EINVAL);
  /* to do: send direction only: must chexk if params->redundancy is correct depending on the payload*/
  if (handle->params.redundancy>2) return(-EINVAL);
  /* allocate the incoming queue */
  if (handle->direction!=RTP_SEND)
    handle->rcv_stack=(q_element_t*)calloc(handle->params.stacksize,sizeof(q_element_t));
  /* setup local address */
        /* for rtp */
  handle->rtp_loc_addr.sin_family=AF_INET;
  handle->rtp_loc_addr.sin_addr.s_addr=inet_addr(params->rtp_loc_addr);
  if (handle->rtp_loc_addr.sin_addr.s_addr==-1) return(-300);
  handle->rtp_loc_addr.sin_port=htons(params->rtp_loc_port);
        /* for rtcp */
  handle->rtcp_loc_addr.sin_family=AF_INET;
  handle->rtcp_loc_addr.sin_addr.s_addr=inet_addr(params->rtp_loc_addr);
  if (handle->rtcp_loc_addr.sin_addr.s_addr==-1) return(-300);
  handle->rtcp_loc_addr.sin_port=htons((uint16_t)(params->rtp_loc_port+1));
  /* setup remote address if necessary*/
  if (handle->direction!=RTP_RECV)
    {
        /*for rtp */
      handle->rtp_rem_addr.sin_family=AF_INET;
      handle->rtp_rem_addr.sin_addr.s_addr=inet_addr(params->rtp_rem_addr);
      if (handle->rtp_rem_addr.sin_addr.s_addr==-1) return(-300);
      handle->rtp_rem_addr.sin_port=htons(params->rtp_rem_port);

        /* for rtcp */
      handle->rtcp_rem_addr.sin_family=AF_INET;
      handle->rtcp_rem_addr.sin_addr.s_addr=inet_addr(params->rtp_rem_addr);
      if (handle->rtcp_rem_addr.sin_addr.s_addr==-1) return(-300);
      handle->rtcp_rem_addr.sin_port=htons((uint16_t)(params->rtp_rem_port+1));

    }
  /* create socket for rtp */
  handle->rtp_socket=socket(PF_INET,SOCK_DGRAM,0);
  if (handle->rtp_socket==-1) return(-11);
  addrlen=sizeof(handle->rtp_loc_addr);
  err=bind(handle->rtp_socket,(struct sockaddr*)&handle->rtp_loc_addr,addrlen);
  if (err==-1) 
    {
      close(handle->rtp_socket);
      return(-EADDRINUSE);
    }
  err=setsockopt(handle->rtp_socket,SOL_SOCKET,SO_REUSEADDR,
		 (void*)&option,sizeof(option));
  if (err==-1) fprintf(stderr,"Warning: The address is not reusable !\n");
  /* set large rcv buffers for sockets*/
  option=65535;
  err=setsockopt(handle->rtp_socket,SOL_SOCKET,SO_RCVBUF,(void*)&option,sizeof(option));
  if (err!=0) fprintf(stderr,"rtplib Warning: the rcvbuf size cannot be set to 65535.\n");
  fcntl(handle->rtp_socket,F_SETFL,O_NONBLOCK);
  if (handle->direction!=RTP_RECV)
    {
      /* connect socket */
      err=connect(handle->rtp_socket,(struct sockaddr*)&handle->rtp_rem_addr,sizeof(handle->rtp_rem_addr));
    }
  /* create socket for rtcp*/
  handle->rtcp_socket=socket(PF_INET,SOCK_DGRAM,0);
  if (handle->rtcp_socket==-1) 
    {
      close(handle->rtp_socket);
      close(handle->rtcp_socket);
      return(-11);
    }
  addrlen=sizeof(handle->rtcp_loc_addr);
  err=bind(handle->rtcp_socket,(struct sockaddr*)&handle->rtcp_loc_addr,addrlen);
  if (err==-1) 
    {
      close(handle->rtp_socket);
      close(handle->rtcp_socket);
      return(-EADDRINUSE);
    }
  err=setsockopt(handle->rtcp_socket,SOL_SOCKET,SO_REUSEADDR,
		 (void*)&option,sizeof(option));
  if (err==-1) fprintf(stderr,"The address is not reusable !\n");
  fcntl(handle->rtcp_socket,F_SETFL,O_NONBLOCK);
  handle->highest_fd=MAX(handle->rtp_socket,handle->rtcp_socket)+1;
  handle->rcv_firsts=1;
  handle->rcv_count=0;
  handle->snd_firsts=1;
  handle->rcv_tsinc=handle->snd_tsinc=get_tsinc(handle->params.profile);
  if (handle->snd_tsinc==-1) return(-EOPNOTSUPP);
  if (params->ssrc==0) params->ssrc=choose_uint32();
  /* allocate and setup the packet generator*/
  if (handle->direction!=RTP_RECV)
    {
      rtp_packet_t *p;
      handle->snd_stack=(q_element_t*)calloc(3,sizeof(q_element_t));
      for(j=0;j<3;j++)
      {
        handle->snd_stack[j].packet_size=0;
        p=&handle->snd_stack[j].packet;
        memset(p,0,sizeof(rtp_packet_t));
        p->version=2;
        p->padbit=0;
        p->extbit=0;
        p->cc=0;
        p->markbit=0;
        p->paytype=handle->params.profile;
        handle->snd_stack[j].p_payload=p->data;
      }
    }
  return(0);
}

int rtp_destroy_session(RtpSession *handle)
{
  close(handle->rtp_socket);
  close(handle->rtcp_socket);

  if (handle->rcv_stack!=NULL) free(handle->rcv_stack);
  if (handle->snd_stack!=NULL) free(handle->snd_stack);
  free(handle);
  return(0);
}

int rtp_debug(RtpSession *handle,FILE *debug_file)
{
  handle->dbg_file=debug_file;
  return(0);
}

int rtp_getstats(RtpSession *handle,int where, rtp_stats_t *stats)
{
	switch(where)
	{
	case RTP_STATS_LOCAL:
		memcpy(stats,&handle->local_stats,sizeof(rtp_stats_t));
		break;
	case RTP_STATS_REMOTE:
		return(-223);
		break;
	default:
		return(-22);
		break;
	}
	return(0);
}


