/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef LINPHONEMAIN_H
#define LINPHONEMAIN_H

#include <stdio.h>
#include <stdint.h>
#include <glib.h>

#include "../osipua/OsipUA.h"
#include "codec.h"

/* version number in hex*/
#define INT_VERSION 0x000400  /*0.4.0*/

/* max number of network interfaces*/
#define MAX_IF 5

/* relative path where is stored local ring*/
#define SOUND_LOCAL_RING "sounds/ring.wav"
/* same for remote ring (ringback)*/
#define SOUND_REMOTE_RING "sounds/ringback.wav"


typedef struct sip_config
{
	char *contact;
	char *username;
	char *hostname;
	FILE *sip_dbg;
	uint16_t sip_port;
} sip_config_t;

typedef struct rtp_config
{
	FILE *rtp_dbg;
	uint16_t audio_rtp_port;
	uint16_t video_rtp_port;
	char jitt_comp;  /*jitter compensation*/
}rtp_config_t;

typedef struct net_config
{
	char if_names[MAX_IF][20];
	char if_addr[MAX_IF][20];
	char *local_addr;  /* the used address that is one somewhere above, but in dot notation*/
	char *if_name;   		/* the used interface name */
	double bandwidth; 		/* the bandwidth in bit/s of the selected interface*/ /*unused*/
	char con_type;			/* type of connection 28k 56k 128k adsl ...*/
	char if_sel;				/* index of the interface used*/
	char if_count;   /* number of interfaces found*/
}net_config_t;


typedef struct sound_config
{
	char *snd_name;   /* name of sound card*/ /*unused*/
	char *snd_dev;    /* /dev/dsp */			/*unused*/
	char rec_lev;
	char play_lev;
	char source;
	int autokill:1;  /* 1= automatically free access to sound device by killing artsd and esd*/
	int driver_mode:2; /* 0=oss 1=alsa-0.5.x 2=alsa-0.9.x (for later use) */
} sound_config_t;

typedef struct codecs_config
{
	GList *audio_codecs;  /* list of audio codecs in order of preference*/
	GList *video_codecs;	/* for later used*/
	GList *codecs;		/*list of codecs, in any order*/
}codecs_config_t;

typedef struct gui_config
{
	GList *address_list;
	int is_daemon;
	int is_applet;
}gui_config_t;

/* this is the main class for linphone, can be used statically*/

typedef struct _LinphoneMain
{
	net_config_t net_conf;
	sip_config_t sip_conf;
	rtp_config_t rtp_conf;
	sound_config_t sound_conf;
	codecs_config_t codecs_conf;
	gui_config_t gui_conf;
	OsipUA *ua;			/* the osip user agent*/
	OsipCallLeg *call;   /* the current call, NULL if no call is running*/
}LinphoneMain;

/* table of different bandwidth for different connection type*/
extern const double bandwidths[5];


/* init object */
void linphone_main_init(LinphoneMain *lpm);

/* register a codec*/
void linphone_main_register_codec(LinphoneMain *lpm,codec_info_t *codec);

/* if there is no config file, use this functions to set default parameters*/
int linphone_main_first_startup(LinphoneMain *lpm);

/* this functions loads linphone configuration from a config file. If the file does not exists, returns -1.*/
int linphone_main_read_config(LinphoneMain *lpm);

/* save current linphone configuration*/
int linphone_main_save_config(LinphoneMain *lpm);

/* call this when the order of codec has changed*/
void linphone_main_apply_codec_change(LinphoneMain *lpm);

/* test if codecs are usable compared to bandwidth.*/
codec_info_t *linphone_main_codecs_check(LinphoneMain *lpm);

/* create the sip user agent and run it*/
int linphone_main_start(LinphoneMain *lpm);

/* stop all processes properly*/
void linphone_main_stop(LinphoneMain *lpm);


/* get the codec_info_t for codec implementing profile */
codec_info_t * linphone_main_get_codec(LinphoneMain *lpm, gint profile);

/***************************************************************************************************************/
/* private functions																																													*/
/**************************************************************************************************************/

/* retrieves interface names and address excluding local loopback. Returns the number of availlable interfaces found.*/
int retrieve_if(LinphoneMain *lpm);

/* kill esd and artsd if they are running, in order to free the sound device access*/
int kill_sound_daemons();




/*       LOCK FILE UTILITIES          */

/* put a lock file in /tmp. this is called when linphone runs as a daemon*/
int set_lock_file();

/* looks if there is a lock file. If presents return its content (the pid of the already running linphone), if not found, returns -1*/
int get_lock_file();

/* remove the lock file if it was set*/
int remove_lock_file();


#endif